@@ -2,14 +2,14 @@ package Google::Adwords::Service;
 use strict;
 use warnings;
 
-use version; our $VERSION = qv('0.10');
+use version; our $VERSION = qv('0.12');
 
 use base qw/ Class::Accessor::Chained Google::Adwords /;
 use SOAP::Lite;
 use Readonly;
 
-Readonly my $default_api_version => 'v10';
-Readonly my $user_agent          => "Google::Adwords v1.5";
+Readonly my $default_api_version => 'v11';
+Readonly my $user_agent          => "Google::Adwords v1.8";
 Readonly my $endpoint            => 'https://adwords.google.com/api/adwords';
 Readonly my $endpoint_sandbox    => 'https://sandbox.google.com/api/adwords';
 Readonly my $default_timeout => 35;    # HTTP timeout in seconds
@@ -276,6 +276,12 @@ sub _create_object_from_hash
 {
     my ( $self, $hashref, $class_name ) = @_;
 
+    # XXX wtf?
+    if ( ref $hashref ne 'HASH' )
+    {
+        $hashref = {};
+    }
+
     my $obj = $class_name->new($hashref);
     return $obj;
 }
